@@ -1,8 +1,8 @@
 <?php
 
 /**
-  * SquirrelMail Virtual Host Login Plugin
-  * Copyright (C) 2003-2005 Paul Lesneiwski <pdontthink@angrynerds.com>
+  * SquirrelMail Login Manager Plugin
+  * Copyright (C) 2003-2006 Paul Lesneiwski <paul@squirrelmail.org>
   * This program is licensed under GPL. See COPYING for details
   *
   * @package plugins
@@ -12,6 +12,58 @@
 
 
 
+/**
+  * Load plugin configuration
+  *
+  * This function loads the correct configuration file.  If a "meta-
+  * configuration" file is present, it is always loaded, along with
+  * the group of settings it points to.  If a custom configuration 
+  * file is present, it is loaded.  If BOTH of these are present, 
+  * note that the custom configuration file settings will override
+  * any "meta-coniguration" if there are overlapping settings.
+  *
+  */
+function load_vlogin_config()
+{
+
+   global $strip_domain_only, $add_default_domain_only;
+   $config_settings_loaded = FALSE;
+
+
+   // load meta config
+   //
+   @include_once (SM_PATH . 'plugins/vlogin/data/meta_config.php');
+
+
+   // strip domain from username only?
+   //
+   if ($strip_domain_only)
+   {
+      include_once(SM_PATH . 'plugins/vlogin/data/meta_config.strip_domain.php');
+      $config_settings_loaded = TRUE;
+   }
+
+
+   // add default domain to username only?
+   //
+   if ($add_default_domain_only)
+   {
+      include_once(SM_PATH . 'plugins/vlogin/data/meta_config.add_default_domain.php');
+      $config_settings_loaded = TRUE;
+   }
+
+
+   // load custom config if present, or load typical settings 
+   // if none found at all
+   //
+   if ((!@include_once(SM_PATH . 'plugins/vlogin/data/config.php'))
+    && !$config_settings_loaded)
+      include_once (SM_PATH . 'plugins/vlogin/data/config.sample.typical.php');
+
+}
+
+
+
 /**
   * Override Default Config Values
   *
@@ -45,7 +97,13 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
           $squirrelmail_plugin_hooks, $useSessionBased, $serviceLevel,
           $usernameDomainIsHost, $at, $stripDomainFromUserSubstitution,
           $SQLDatabaseDomainLookup, $SQLDatabaseServiceLevelLookup,
-          $default_org_logo;
+          $default_org_logo, $notPartOfDomainName, 
+          $chopOffDotSectionsFromRight, $chopOffDotSectionsFromLeft, 
+          $numberOfDotSections, $checkByExcludeList, $dot, 
+          $replacements, $removeFromFront, $translateHostnameTable, 
+          $pathToQmail, $reverseDotSectionOrder,
+          $always_prepend, $always_append, $IMAPServerRules, 
+          $SMTPServerRules;
       
   
   // get global variable for versions of PHP < 4.1 
@@ -95,8 +153,7 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
   {
 
      $checkServiceLevel = TRUE;
-     if (!@include_once (SM_PATH . 'plugins/vlogin/data/config.php'))
-        include_once (SM_PATH . 'plugins/vlogin/data/config.php.sample.typical');
+     load_vlogin_config();
 
 
      // prep: get config settings from session if necessary
@@ -144,7 +201,18 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
      }
    
    
-     $hostname_stripped = deconstructDomainName($hostname);
+     $hostname_stripped = deconstructDomainName($hostname,
+                                                $notPartOfDomainName, 
+                                                $chopOffDotSectionsFromRight, 
+                                                $chopOffDotSectionsFromLeft, 
+                                                $numberOfDotSections, 
+                                                $checkByExcludeList, 
+                                                $at, $dot, $replacements,
+                                                $removeFromFront, 
+                                                $translateHostnameTable, 
+                                                $pathToQmail,
+                                                $reverseDotSectionOrder,
+                                                $always_prepend, $always_append);
      $orig_hostname_stripped = $hostname_stripped;
    
    
@@ -166,7 +234,152 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
      if ($usernameDomainIsHost && strpos($user, $at) !== FALSE ) 
      {
         $hostname = substr($user, strpos($user, $at) + strlen($at));
-        $hostname_stripped = deconstructDomainName($hostname);
+        $hostname_stripped = deconstructDomainName($hostname,
+                                                   $notPartOfDomainName, 
+                                                   $chopOffDotSectionsFromRight, 
+                                                   $chopOffDotSectionsFromLeft, 
+                                                   $numberOfDotSections, 
+                                                   $checkByExcludeList, 
+                                                   $at, $dot, $replacements,
+                                                   $removeFromFront, 
+                                                   $translateHostnameTable, 
+                                                   $pathToQmail,
+                                                   $reverseDotSectionOrder,
+                                                   $always_prepend, $always_append);
+     }
+
+
+     // determine IMAP server address if it should be based 
+     // on the user's domain
+     //
+     if (!empty($IMAPServerRules))
+     {
+
+        if (strpos($user, $at) !== FALSE) 
+           $imap_domain = substr($user, strpos($user, $at) + strlen($at));
+        else 
+           $imap_domain = $hostname_stripped;
+
+        $server_addr = deconstructDomainName($imap_domain,
+                 (empty($IMAPServerRules['notPartOfDomainName']) 
+                  ? array() 
+                  : $IMAPServerRules['notPartOfDomainName']), 
+                 (empty($IMAPServerRules['chopOffDotSectionsFromRight'])
+                  ? 0
+                  : $IMAPServerRules['chopOffDotSectionsFromRight']),
+                 (empty($IMAPServerRules['chopOffDotSectionsFromLeft'])
+                  ? 0
+                  : $IMAPServerRules['chopOffDotSectionsFromLeft']),
+                 (empty($IMAPServerRules['numberOfDotSections'])
+                  ? 0
+                  : $IMAPServerRules['numberOfDotSections']),
+                 (empty($IMAPServerRules['checkByExcludeList'])
+                  ? 0
+                  : $IMAPServerRules['checkByExcludeList']),
+                 (empty($IMAPServerRules['at'])
+                  ? ''
+                  : $IMAPServerRules['at']),
+                 (empty($IMAPServerRules['dot'])
+                  ? ''
+                  : $IMAPServerRules['dot']),
+                 (empty($IMAPServerRules['replacements'])
+                  ? array()
+                  : $IMAPServerRules['replacements']),
+                 (empty($IMAPServerRules['removeFromFront'])
+                  ? 0
+                  : $IMAPServerRules['removeFromFront']),
+                 (empty($IMAPServerRules['translateHostnameTable'])
+                  ? ''
+                  : $IMAPServerRules['translateHostnameTable']),
+                 (empty($IMAPServerRules['pathToQmail'])
+                  ? ''
+                  : $IMAPServerRules['pathToQmail']),
+                 (empty($IMAPServerRules['reverseDotSectionOrder'])
+                  ? 0
+                  : $IMAPServerRules['reverseDotSectionOrder']),
+                 (empty($IMAPServerRules['always_prepend'])
+                  ? ''
+                  : $IMAPServerRules['always_prepend']),
+                 (empty($IMAPServerRules['always_append'])
+                  ? ''
+                  : $IMAPServerRules['always_append'])
+                                            );
+
+        if ($useSessionBased)
+           $config_override['imapServerAddress'] = $server_addr;
+        else
+        {
+           global $imapServerAddress;
+           $imapServerAddress = $server_addr;
+        }
+
+     }
+
+
+     // determine SMTP server address if it should be based
+     // on the user's domain
+     //
+     if (!empty($SMTPServerRules))
+     {
+
+        if (strpos($user, $at) !== FALSE)
+           $smtp_domain = substr($user, strpos($user, $at) + strlen($at));
+        else
+           $smtp_domain = $hostname_stripped;
+
+        $server_addr = deconstructDomainName($smtp_domain,
+                 (empty($SMTPServerRules['notPartOfDomainName'])
+                  ? array()
+                  : $SMTPServerRules['notPartOfDomainName']),
+                 (empty($SMTPServerRules['chopOffDotSectionsFromRight'])
+                  ? 0
+                  : $SMTPServerRules['chopOffDotSectionsFromRight']),
+                 (empty($SMTPServerRules['chopOffDotSectionsFromLeft'])
+                  ? 0
+                  : $SMTPServerRules['chopOffDotSectionsFromLeft']),
+                 (empty($SMTPServerRules['numberOfDotSections'])
+                  ? 0
+                  : $SMTPServerRules['numberOfDotSections']),
+                 (empty($SMTPServerRules['checkByExcludeList'])
+                  ? 0
+                  : $SMTPServerRules['checkByExcludeList']),
+                 (empty($SMTPServerRules['at'])
+                  ? ''
+                  : $SMTPServerRules['at']),
+                 (empty($SMTPServerRules['dot'])
+                  ? ''
+                  : $SMTPServerRules['dot']),
+                 (empty($SMTPServerRules['replacements'])
+                  ? array()
+                  : $SMTPServerRules['replacements']),
+                 (empty($SMTPServerRules['removeFromFront'])
+                  ? 0
+                  : $SMTPServerRules['removeFromFront']),
+                 (empty($SMTPServerRules['translateHostnameTable'])
+                  ? ''
+                  : $SMTPServerRules['translateHostnameTable']),
+                 (empty($SMTPServerRules['pathToQmail'])
+                  ? ''
+                  : $SMTPServerRules['pathToQmail']),
+                 (empty($SMTPServerRules['reverseDotSectionOrder'])
+                  ? 0
+                  : $SMTPServerRules['reverseDotSectionOrder']),
+                 (empty($SMTPServerRules['always_prepend'])
+                  ? ''
+                  : $SMTPServerRules['always_prepend']),
+                 (empty($SMTPServerRules['always_append'])
+                  ? ''
+                  : $SMTPServerRules['always_append'])
+                                            );
+
+        if ($useSessionBased)
+           $config_override['smtpServerAddress'] = $server_addr;
+        else
+        {
+           global $smtpServerAddress;
+           $smtpServerAddress = $server_addr;
+        }
+
      }
 
 
@@ -224,8 +437,9 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
       {
          echo '<html><body><font color="red" size="12pt">';
          echo 'Sorry, please contact your administrator and ';
-         echo 'ask them to reconfigure the SquirrelMail Virtual Host Login ';
-         echo 'plugin such that the global virtual domain is listed first';
+         echo 'ask them to reconfigure the SquirrelMail ';
+         echo 'Login Manager (vlogin) plugin such that the ';
+         echo 'global virtual domain is listed first';
          echo '</font></body></html>';
          exit;
       }
@@ -314,9 +528,9 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
               if (isset($virtualDomains[$virtualDomain]['settingsWithEmbeddedPHP']) 
                && is_array($virtualDomains[$virtualDomain]['settingsWithEmbeddedPHP'])
                && in_array($setting, $virtualDomains[$virtualDomain]['settingsWithEmbeddedPHP']))
-                 $$setting = $value;
-              else
                  eval('$$setting = ' . $value . ';');
+              else
+                 $$setting = $value;
 
               $config_override[$setting] = $value;
            }
@@ -449,13 +663,18 @@ function determine_user_hostname()
 {
 
    global $useDomainFromVirtDomainsArray, $virtualDomains, $securePort,
-          $allVirtualDomainsAreUnderOneHost, $forceLowercase;
+          $allVirtualDomainsAreUnderOneHost, $forceLowercase,
+          $useDomainFromServerEnvironment, $vlogin_debug,
+          $notPartOfDomainName, $chopOffDotSectionsFromRight, 
+          $chopOffDotSectionsFromLeft, $numberOfDotSections, 
+          $checkByExcludeList, $at, $dot, $replacements,
+          $removeFromFront, $translateHostnameTable, $pathToQmail, 
+          $reverseDotSectionOrder, $always_prepend, $always_append;
 
 
    // get vlogin config settings
    //
-   if (!@include_once (SM_PATH . 'plugins/vlogin/data/config.php'))
-      include_once (SM_PATH . 'plugins/vlogin/data/config.php.sample.typical');
+   load_vlogin_config();
 
 
    // get global variable for versions of PHP < 4.1
@@ -498,33 +717,51 @@ function determine_user_hostname()
   // the actual host name out of the PHP_SELF portion of the URL
   //
 // old way:
-//  if (isset($_SERVER['HTTPS']) 
+//   if (isset($_SERVER['HTTPS']) 
 // um, why restrict this to only https?
-//  if ($serverPort == $httpsPort
-//   && $allVirtualDomainsAreUnderOneHost) 
-  if ($allVirtualDomainsAreUnderOneHost) 
-  {
+//   if ($serverPort == $httpsPort
+//    && $allVirtualDomainsAreUnderOneHost) 
+   if ($allVirtualDomainsAreUnderOneHost) 
+   {
 
-     // old way, static location of domain on end of URI:
-     // preg_match('/[\/]*(.*?)(\/|$)/', $_SERVER['REQUEST_URI'], $matches);
-     preg_match($allVirtualDomainsAreUnderOneHost, $_SERVER['REQUEST_URI'], $matches);
-     $hostname = $matches[1];
+      // old way, static location of domain on end of URI:
+      // preg_match('/[\/]*(.*?)(\/|$)/', $_SERVER['REQUEST_URI'], $matches);
+      preg_match($allVirtualDomainsAreUnderOneHost, $_SERVER['REQUEST_URI'], $matches);
+      $hostname = $matches[1];
 
-  }
+   }
 
 
 
-  // go lowercase if needed
-  //
-  if ($forceLowercase)
-     $hostname = strtolower($hostname);
+   $found = FALSE;
+
+
+
+   // use server name from server environment if necessary
+   //
+   if (!empty($useDomainFromServerEnvironment))
+   {
+
+      $hostname = $_SERVER[$useDomainFromServerEnvironment];
+      $found = TRUE;
+
+      if ($vlogin_debug) 
+         echo 'Using hostname from SERVER environment... $_SERVER[\'' 
+            . $useDomainFromServerEnvironment . '\'] = ' 
+            . $hostname . '<br />';
+   }
+
+
+   // go lowercase if needed
+   //
+   if ($forceLowercase)
+      $hostname = strtolower($hostname);
 
 
 
    // allow the use of the domain setting from the $virtualDomains
    // array, which is a heck of a lot simpler...
    //
-   $found = FALSE;
    if ($useDomainFromVirtDomainsArray)
 // TODO: Do this for SQL too??  We might need another query that just gets the domain and use it here.  Yuck.
    {
@@ -552,7 +789,18 @@ function determine_user_hostname()
       }
 
    }
-   if (!$found) $hostname = deconstructDomainName($hostname);
+   if (!$found) $hostname = deconstructDomainName($hostname,
+                                                  $notPartOfDomainName, 
+                                                  $chopOffDotSectionsFromRight, 
+                                                  $chopOffDotSectionsFromLeft, 
+                                                  $numberOfDotSections, 
+                                                  $checkByExcludeList, 
+                                                  $at, $dot, $replacements,
+                                                  $removeFromFront, 
+                                                  $translateHostnameTable, 
+                                                  $pathToQmail,
+                                                  $reverseDotSectionOrder,
+                                                  $always_prepend, $always_append);
 
 
    return $hostname;
@@ -579,7 +827,15 @@ function vlogin_domain_do($args)
          $sendmailVirtualUserTable, $putHostNameOnFrontOfUsername,
          $prefs_dsn, $useSessionBased, $removeDomainIfGiven, 
          $alwaysAddHostName, $usernameReplacements, $forceLowercase,
-         $prefs_are_cached, $prefs_cache;
+         $prefs_are_cached, $prefs_cache, $vlogin_remap_username_completed,
+         $dontUseHostNameUserList;
+
+
+  // we've already been here.... don't do this twice
+  // (used for compatibility/interoperability with
+  // other plugins)
+  //
+  if ($vlogin_remap_username_completed) return;
 
 
   // figure out where prefs are stored
@@ -592,8 +848,7 @@ function vlogin_domain_do($args)
 
   // get vlogin config settings
   //
-  if (!@include_once (SM_PATH . 'plugins/vlogin/data/config.php'))
-     include_once (SM_PATH . 'plugins/vlogin/data/config.php.sample.typical');
+  load_vlogin_config();
 
 
 
@@ -624,6 +879,23 @@ function vlogin_domain_do($args)
 
 
 
+  // save the name originally used as the login (gets saved in user
+  // session on the login_verified hook)... might be useful later
+  //
+  global $vlogin_original_username;
+  $vlogin_original_username = $user;
+
+
+
+  // if original username is in the $dontUseHostNameUserList array,
+  // make sure $dontUseHostName is turned on
+  //
+  if (is_array($dontUseHostNameUserList) 
+   && in_array($vlogin_original_username, $dontUseHostNameUserList))
+     $dontUseHostName = 1;
+
+
+
   // if user logged in with full email address, 
   // chop off domain info if needed
   //
@@ -661,7 +933,7 @@ function vlogin_domain_do($args)
 
       // check for login alias here and return if found
       // (but only if it is in the domain being used
-      // to log in when dontUseHostName is off)
+      // to log in when $dontUseHostName is off)
       //
       $prefs_are_cached = false;
       $prefs_cache = array();
@@ -698,6 +970,7 @@ function vlogin_domain_do($args)
 
   // convert "at" sign...
   //
+  if (is_array($atConversion))
   foreach ($atConversion as $otherAt)
      $user = str_replace($otherAt, $at, $user);
 
@@ -757,6 +1030,11 @@ function vlogin_domain_do($args)
      $login_username=$realname;
 
 
+  // tell anyone else who might be looking that we're done
+  //
+  $vlogin_remap_username_completed = TRUE;
+
+
   // when in debug mode, just dump out final login name and quit
   //
   if ($vlogin_debug)
@@ -773,7 +1051,7 @@ function vlogin_domain_do($args)
      echo '<br>PHP_SELF is ' . (isset($_SERVER['PHP_SELF']) ? $_SERVER['PHP_SELF'] : 'not globally accessible')
         . '<br>';
 global        $imapServerAddress;
-echo '<hr>' . $imapServerAddress . '<hr>';
+echo '<hr>IMAP server: ' . $imapServerAddress . '<hr>';
      echo '<br><hr></body></html>';
      exit;
 
@@ -783,6 +1061,32 @@ echo '<hr>' . $imapServerAddress . '<hr>';
 
 
 
+/**
+  * Save original username used to log in
+  *
+  */
+function vlogin_save_original_do()
+{
+
+   // get vlogin config settings
+   //
+   load_vlogin_config();
+
+
+   global $vlogin_original_username, $rawLoginIsOutgoingEmailAddress, 
+          $username, $data_dir;
+   sqsession_register($vlogin_original_username, 'vlogin_original_username');
+
+
+   // force outgoing email address?
+   //
+   if ($rawLoginIsOutgoingEmailAddress)
+      setPref($data_dir, $username, 'email_address', $vlogin_original_username);
+
+}
+
+
+
 /** 
   * Get Per-User Config Overrides
   *
@@ -899,26 +1203,40 @@ function serviceLevelOverride($user)
   * Host Name Deconstruction
   *
   * Takes a host name (usually from URL, but can be
-  * anything) and reconstructs it based on vlogin
+  * anything) and reconstructs it based on the given
   * configuration settings.
   *
   * @param string $hostname The hostname to re/deconstruct.
+  * TODO: fix param documentation; see README or config files
+  * @param $notPartOfDomainName 
+  * @param $chopOffDotSectionsFromRight 
+  * @param $chopOffDotSectionsFromLeft 
+  * @param $numberOfDotSections
+  * @param $checkByExcludeList 
+  * @param $at
+  * @param $dot
+  * @param $replacements
+  * @param $removeFromFront
+  * @param $translateHostnameTable 
+  * @param $pathToQmail
+  * @param $reverseDotSectionOrder
+  * @param $always_prepend
+  * @param $always_append
   *
   * @return string The rebuilt hostname.
   *
   */
-function deconstructDomainName($hostname) 
+function deconstructDomainName($hostname, $notPartOfDomainName, 
+                               $chopOffDotSectionsFromRight, 
+                               $chopOffDotSectionsFromLeft, 
+                               $numberOfDotSections, $checkByExcludeList, 
+                               $at, $dot, $replacements,
+                               $removeFromFront, $translateHostnameTable, 
+                               $pathToQmail, $reverseDotSectionOrder,
+                               $always_prepend, $always_append)
 {
 
-  global $notPartOfDomainName, $chopOffDotSectionsFromRight, 
-         $chopOffDotSectionsFromLeft, $numberOfDotSections, 
-         $checkByExcludeList, $at, $dot, $replacements,
-         $removeFromFront, $translateHostnameTable, $pathToQmail,
-         $reverseDotSectionOrder;
-
-
-  if (!@include_once (SM_PATH . 'plugins/vlogin/data/config.php'))
-     include_once (SM_PATH . 'plugins/vlogin/data/config.php.sample.typical');
+  load_vlogin_config();
 
 
 
@@ -1004,9 +1322,16 @@ function deconstructDomainName($hostname)
 
 
   // just in case they need a different '.' seperator
+  //
   $hostname = str_replace( ".", $dot, $hostname );
 
 
+  // append or prepend static strings if given
+  //
+  if (!empty($always_prepend)) $hostname = $always_prepend . $hostname;
+  if (!empty($always_append)) $hostname = $hostname . $always_append;
+
+
   // if domain name needs to be remapped, do so here
   //
   if (!empty($translateHostnameTable))
@@ -1085,9 +1410,11 @@ function translateHostname($host, $translateHostnameTable)
          $line = trim($line);
 
 
-         // skip blank lines and comment lines
+         // skip blank lines and comment lines and php
+         // open/close tag lines 
          //
-         if (strpos($line, '#') === 0 || strlen($line) < 3)
+         if (strpos($line, '#') === 0 || strlen($line) < 3 
+          || strpos($line, '<?php') === 0 || strpos($line, '*/ ?>') === 0)
             continue;
 
 
@@ -1165,9 +1492,11 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
          $line = trim($line);
 
 
-         // skip blank lines and comment lines
+         // skip blank lines and comment lines and php
+         // open/close tag lines 
          //
-         if (strpos($line, '#') === 0 || strlen($line) < 3)
+         if (strpos($line, '#') === 0 || strlen($line) < 3 
+          || strpos($line, '<?php') === 0 || strpos($line, '*/ ?>') === 0)
             continue;
 
 
@@ -1252,7 +1581,12 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
 function vlogin_display_options_do()
 {
 
-   global $config_override, $optpage_data;
+   global $config_override, $optpage_data, $notPartOfDomainName, 
+          $chopOffDotSectionsFromRight, $chopOffDotSectionsFromLeft, 
+          $numberOfDotSections, $checkByExcludeList, $at, $dot, 
+          $replacements, $removeFromFront, $translateHostnameTable, 
+          $pathToQmail, $reverseDotSectionOrder, $always_prepend, 
+          $always_append;
 
    $overrides = array();
 
@@ -1263,8 +1597,7 @@ function vlogin_display_options_do()
    {
 
       global $virtualDomains, $useSessionBased;
-      if (!@include_once (SM_PATH . 'plugins/vlogin/data/config.php'))
-         include_once (SM_PATH . 'plugins/vlogin/data/config.php.sample.typical');
+      load_vlogin_config();
 
       if (!$useSessionBased)
       {
@@ -1284,7 +1617,18 @@ function vlogin_display_options_do()
             $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
          else
             $hostname = $_SERVER['HTTP_HOST'];
-         $hostname_stripped = deconstructDomainName($hostname);
+         $hostname_stripped = deconstructDomainName($hostname,
+                                                    $notPartOfDomainName, 
+                                                    $chopOffDotSectionsFromRight, 
+                                                    $chopOffDotSectionsFromLeft, 
+                                                    $numberOfDotSections, 
+                                                    $checkByExcludeList, 
+                                                    $at, $dot, $replacements,
+                                                    $removeFromFront, 
+                                                    $translateHostnameTable, 
+                                                    $pathToQmail,
+                                                    $reverseDotSectionOrder,
+                                                    $always_prepend, $always_append);
 
 
          // allow inclusion of external configuration files for each domain
